#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>

#include "crank.h"
#include "error.h"
#include "gui_generic_callbacks.h"
#include "gui_generic_interface.h"
#include "componant.h"
#include "parameter.h"
#include "gui_globals.h"
#include "gui_init.h"

void
on_gui_generic_clist_select_row        (GtkCList        *clist,
                                        gint             row,
                                        gint             column,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
    instance *i = (instance *) gtk_object_get_data(GTK_OBJECT(clist), "instance");
    parameter_description *parameter_table = i->componant_class->parameter_description_table;
    parameter_type type = parameter_table[row].type;
    GtkWidget *gui_generic_title_data_one = (GtkWidget *) gtk_object_get_data(GTK_OBJECT(clist), "gui_generic_title_data_one");
    GtkWidget *gui_generic_title_data_two = (GtkWidget *) gtk_object_get_data(GTK_OBJECT(clist), "gui_generic_title_data_two");
    GtkWidget *gui_generic_description = (GtkWidget *) gtk_object_get_data(GTK_OBJECT(clist), "gui_generic_description");
    GtkWidget *gui_generic_data_one = (GtkWidget *) gtk_object_get_data(GTK_OBJECT(clist), "gui_generic_data_one");
    GtkWidget *gui_generic_data_two = (GtkWidget *) gtk_object_get_data(GTK_OBJECT(clist), "gui_generic_data_two");
    GtkWidget *gui_generic_type = (GtkWidget *) gtk_object_get_data(GTK_OBJECT(clist), "gui_generic_type");
    GtkWidget *gui_generic_table = (GtkWidget *) gtk_object_get_data(GTK_OBJECT(clist), "gui_generic_table");
    GtkWidget *change_button = (GtkWidget *) gtk_object_get_data(GTK_OBJECT(clist), "change_button");

    gtk_object_set_data(GTK_OBJECT(change_button), "row", GINT_TO_POINTER(row));

    gtk_widget_set_sensitive(GTK_WIDGET(gui_generic_table), TRUE);
    gtk_label_set_text(GTK_LABEL(gui_generic_type), parameter_type_lookup(type));
    gtk_label_set_text(GTK_LABEL(gui_generic_description), parameter_table[row].description);
    switch (type) {
    case PARAM_TYPE_ENUM:
	gtk_label_set_text(GTK_LABEL(gui_generic_title_data_one), "Possible values:");
	gtk_label_set_text(GTK_LABEL(gui_generic_title_data_two), "Number of possible values:");
	break;
    case PARAM_TYPE_INT:
    case PARAM_TYPE_FLOAT:
	gtk_label_set_text(GTK_LABEL(gui_generic_title_data_one), "Minimum value:");
	gtk_label_set_text(GTK_LABEL(gui_generic_title_data_two), "Maximum value:");
	break;
    case PARAM_TYPE_STRING:
	gtk_label_set_text(GTK_LABEL(gui_generic_title_data_one), "");
	gtk_label_set_text(GTK_LABEL(gui_generic_title_data_two), "");
	break;
    default:
	warn("Unknown parameter type %d\n", type);
	break;
    }
    gtk_label_set_text(GTK_LABEL(gui_generic_data_one), parameter_table[row].data_one);
    gtk_label_set_text(GTK_LABEL(gui_generic_data_two), parameter_table[row].data_two);

}


void
on_gui_generic_button_clicked          (GtkButton       *button,
                                        gpointer         user_data)
{
    gchar *text;
    GtkWidget *entry = (GtkWidget *) gtk_object_get_data(GTK_OBJECT(button), "entry");
    instance *i =  (instance *) gtk_object_get_data(GTK_OBJECT(button), "instance");
    int row = GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(button), "row"));
    int intvalue;
    float floatvalue;
    text = gtk_editable_get_chars(GTK_EDITABLE(entry), 0, -1);
    switch (i->componant_class->parameter_description_table[row].type) {

    /* Error checking needed on user data!! */
    case PARAM_TYPE_ENUM:
    case PARAM_TYPE_INT:
	intvalue = atoi(text);
	poke_param(i, row, &intvalue);
	break;
    case PARAM_TYPE_FLOAT:
	floatvalue = atof(text);
	poke_param(i, row, &floatvalue);
	break;
    case PARAM_TYPE_STRING:
	poke_param(i, row, text);
	break;
    default:
	warn("Unknown parameter type\n");
	break;
    }
    rebuild_componant_gui(get_main_window());
    g_free(text);
}

