/* This program is part of Crank, a cryptanalysis tool
 * Copyright (C) 2000 Matthew Russell
 *
 * This program is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License (LICENSE) for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

/* FILE: 
 * gui_init.c
 *
 * FUNCTION: 
 * Functions for building the graphic user interface
 *
 */

#include "gui_init.h"
#include "gui_interface.h"
#include "gui_support.h"
#include "gui_globals.h"
#include "gui_textarea.h"
#include "globals.h"

#define BUFFER_SIZE 1024

void handle_source_text_change(GtkWidget *main_window) {
    GtkWidget *statusbar;
    float fitness_value;
    char *text, *transformed_text, buf[BUFFER_SIZE];
    statusbar = lookup_widget(GTK_WIDGET(main_window), "statusbar");

    text = get_source_text();
    transformed_text = transform(get_current_transform(), text);

    set_view_text(transformed_text);
    
    fitness_value = fitness(get_current_fitness(), transformed_text);
    sprintf(buf, "Fitness: %f", fitness_value);
    gtk_statusbar_pop(GTK_STATUSBAR(statusbar), gtk_statusbar_get_context_id(GTK_STATUSBAR(statusbar), "handle_source_text_change"));
    gtk_statusbar_push(GTK_STATUSBAR(statusbar), gtk_statusbar_get_context_id(GTK_STATUSBAR(statusbar), "handle_source_text_change"), buf);
    
    free(text);
    free(transformed_text);

}

void rebuild_componant_gui(GtkWidget *main_window) {

    instance *initial_transform, *initial_heuristic, *initial_fitness;
    instance *initial_gui_transform, *initial_gui_heuristic, *initial_gui_fitness;
    GtkWidget *transform_frame, *heuristic_frame, *fitness_frame;
    GtkWidget *transform_componant_entry, *heuristic_componant_entry, *fitness_componant_entry;
    GtkWidget *transform_interface_entry, *heuristic_interface_entry, *fitness_interface_entry;
    GtkWidget *temp_widget;
    char buf[BUFFER_SIZE];

    /* Find widgets */
    transform_frame = lookup_widget(GTK_WIDGET(main_window), "transform_frame"); 
    heuristic_frame = lookup_widget(GTK_WIDGET(main_window), "heuristic_frame");
    fitness_frame = lookup_widget(GTK_WIDGET(main_window), "fitness_frame");
    transform_componant_entry = lookup_widget(GTK_WIDGET(main_window), "transform_componant_entry");
    heuristic_componant_entry = lookup_widget(GTK_WIDGET(main_window), "heuristic_componant_entry");
    fitness_componant_entry = lookup_widget(GTK_WIDGET(main_window), "fitness_componant_entry");
    transform_interface_entry = lookup_widget(GTK_WIDGET(main_window), "transform_interface_entry");
    heuristic_interface_entry = lookup_widget(GTK_WIDGET(main_window), "heuristic_interface_entry");
    fitness_interface_entry = lookup_widget(GTK_WIDGET(main_window), "fitness_interface_entry");

    /* Get current instances */
    initial_transform = get_current_transform();
    initial_heuristic = get_current_heuristic();
    initial_fitness = get_current_fitness();
    initial_gui_transform = get_current_gui_transform();
    initial_gui_heuristic = get_current_gui_heuristic();
    initial_gui_fitness = get_current_gui_fitness();

    /* Make initial componant gui widgets */
    temp_widget = build_gui_item(initial_gui_transform, initial_transform);
    gtk_widget_show(temp_widget);
    if (get_current_transform_gui())
	gtk_container_remove(GTK_CONTAINER(transform_frame), get_current_transform_gui());
    set_current_transform_gui(temp_widget);
    gtk_container_add(GTK_CONTAINER(transform_frame), temp_widget);

    temp_widget = build_gui_item(initial_gui_heuristic, initial_heuristic);
    gtk_widget_show(temp_widget);
    if (get_current_heuristic_gui())
	gtk_container_remove(GTK_CONTAINER(heuristic_frame), get_current_heuristic_gui());
    set_current_heuristic_gui(temp_widget);
    gtk_container_add(GTK_CONTAINER(heuristic_frame), temp_widget);

    temp_widget = build_gui_item(initial_gui_fitness, initial_fitness);
    gtk_widget_show(temp_widget);
    if (get_current_fitness_gui())
	gtk_container_remove(GTK_CONTAINER(fitness_frame), get_current_fitness_gui());
    set_current_fitness_gui(temp_widget);
    gtk_container_add(GTK_CONTAINER(fitness_frame), temp_widget);

    /* Set the GTK_ENTRY box texts */
    sprintf(buf, "%s_%s", componant_type_lookup(initial_transform->componant_class->type), initial_transform->componant_class->name);
    gtk_entry_set_text(GTK_ENTRY(transform_componant_entry), buf);
    sprintf(buf, "%s_%s", componant_type_lookup(initial_heuristic->componant_class->type), initial_heuristic->componant_class->name);
    gtk_entry_set_text(GTK_ENTRY(heuristic_componant_entry), buf);
    sprintf(buf, "%s_%s", componant_type_lookup(initial_fitness->componant_class->type), initial_fitness->componant_class->name);
    gtk_entry_set_text(GTK_ENTRY(fitness_componant_entry), buf);

    sprintf(buf, "%s_%s", componant_type_lookup(initial_gui_transform->componant_class->type), initial_gui_transform->componant_class->name);
    gtk_entry_set_text(GTK_ENTRY(transform_interface_entry), buf);
    sprintf(buf, "%s_%s", componant_type_lookup(initial_gui_heuristic->componant_class->type), initial_gui_heuristic->componant_class->name);
    gtk_entry_set_text(GTK_ENTRY(heuristic_interface_entry), buf);
    sprintf(buf, "%s_%s", componant_type_lookup(initial_gui_fitness->componant_class->type), initial_gui_fitness->componant_class->name);
    gtk_entry_set_text(GTK_ENTRY(fitness_interface_entry), buf);
    
    handle_source_text_change(main_window);
    
}

void init_gui(int argc, char *argv[], componant *initial_transform_componant, 
                                             componant *initial_heuristic_componant, 
                                             componant *initial_fitness_componant, 
                                             componant *initial_gui_componant) {
    instance *initial_transform, *initial_heuristic, *initial_fitness;
    instance *initial_gui_transform, *initial_gui_heuristic, *initial_gui_fitness;

    GtkWidget *main_window;

    add_pixmap_directory(DATADIR);
    init_gui_globals();

    /* Initiate GTK+, and create main window */
    gtk_init(&argc, &argv);
    main_window = create_main_window();
    set_main_window(main_window);
    
    /* Setup textareas */
    set_source_text_gui(lookup_widget(GTK_WIDGET(main_window), "source_text"));
    set_view_text_gui(lookup_widget(GTK_WIDGET(main_window), "view_text"));
    
    /* Make initial instances */
    initial_transform = make_new_instance(initial_transform_componant);
    initial_heuristic = make_new_instance(initial_heuristic_componant);
    initial_fitness = make_new_instance(initial_fitness_componant);
    initial_gui_transform = make_new_instance(initial_gui_componant);
    initial_gui_heuristic = make_new_instance(initial_gui_componant);
    initial_gui_fitness = make_new_instance(initial_gui_componant);

    /* Set the global variables */
    set_current_transform(initial_transform);
    set_current_heuristic(initial_heuristic);
    set_current_fitness(initial_fitness);
    set_current_gui_transform(initial_gui_transform);
    set_current_gui_heuristic(initial_gui_heuristic);
    set_current_gui_fitness(initial_gui_fitness);

    rebuild_componant_gui(main_window);
    gtk_widget_show(main_window);
    gtk_main();

}
